-- =============================================
-- SaveToDB Framework for SQLite
-- Version 7.19, March 12, 2018
--
-- This script copies settings from SaveToDB Framework 5-7 to SaveToDB Framework 8.
-- SaveToDB Framework 8 must be installed.
-- You may remove SaveToDB Framework 5-7 manually after successful copying.
--
-- Copyright 2018 Gartle Technology Corporation
-- =============================================

-- print Copying table data:

-- print QueryList         -> objects

INSERT INTO objects
    ( TABLE_SCHEMA
    , TABLE_NAME
    , TABLE_TYPE
    , TABLE_CODE
    , INSERT_OBJECT
    , UPDATE_OBJECT
    , DELETE_OBJECT
    )
SELECT
    s.TABLE_SCHEMA
    , s.TABLE_NAME
    , s.TABLE_TYPE
    , s.TABLE_CODE
    , s.INSERT_PROCEDURE
    , s.UPDATE_PROCEDURE
    , s.DELETE_PROCEDURE
FROM
    QueryList s
    LEFT OUTER JOIN objects t ON t.TABLE_NAME = s.TABLE_NAME
WHERE
    NOT s.TABLE_NAME IN ('ColumnTranslation', 'ObjectTranslation', 'TableFormats', 'EventHandlers', 'ParameterValues', 'QueryList')
    AND t.ID IS NULL;

-- print EventHandlers     -> handlers

INSERT INTO handlers
    ( TABLE_SCHEMA
    , TABLE_NAME
    , COLUMN_NAME
    , EVENT_NAME
    , HANDLER_SCHEMA
    , HANDLER_NAME
    , HANDLER_TYPE
    , HANDLER_CODE
    , TARGET_WORKSHEET
    , MENU_ORDER
    , EDIT_PARAMETERS
    )
SELECT
    s.TABLE_SCHEMA
    , s.TABLE_NAME
    , s.COLUMN_NAME
    , s.EVENT_NAME
    , s.HANDLER_SCHEMA
    , s.HANDLER_NAME
    , s.HANDLER_TYPE
    , s.HANDLER_CODE
    , s.TARGET_WORKSHEET
    , s.MENU_ORDER
    , s.EDIT_PARAMETERS
FROM
    EventHandlers s
    LEFT OUTER JOIN handlers t ON t.TABLE_NAME = s.TABLE_NAME
        AND COALESCE(t.COLUMN_NAME, '')     = COALESCE(s.COLUMN_NAME, '')
        AND COALESCE(t.HANDLER_SCHEMA, '')  = COALESCE(s.HANDLER_SCHEMA, '')
        AND COALESCE(t.HANDLER_NAME, '')    = COALESCE(s.HANDLER_NAME, '')
        AND COALESCE(t.HANDLER_TYPE, '')    = COALESCE(s.HANDLER_TYPE, '')
        AND COALESCE(t.MENU_ORDER, 0)       = COALESCE(s.MENU_ORDER, 0)
WHERE
    NOT s.TABLE_NAME IN ('ColumnTranslation', 'ObjectTranslation', 'TableFormats', 'EventHandlers', 'ParameterValues', 'QueryList')
    AND t.ID IS NULL;

-- print ParameterValues   -> handlers

INSERT INTO handlers
    ( TABLE_SCHEMA
    , TABLE_NAME
    , COLUMN_NAME
    , EVENT_NAME
    , HANDLER_SCHEMA
    , HANDLER_NAME
    , HANDLER_TYPE
    , HANDLER_CODE
    , TARGET_WORKSHEET
    , MENU_ORDER
    , EDIT_PARAMETERS
    )
SELECT
    s.TABLE_SCHEMA
    , s.TABLE_NAME
    , s.PARAMETER_NAME  AS COLUMN_NAME
    , 'ParameterValues' AS EVENT_NAME
    , s.SELECT_SCHEMA   AS HANDLER_SCHEMA
    , s.SELECT_NAME     AS HANDLER_NAME
    , s.SELECT_TYPE     AS HANDLER_TYPE
    , s.SELECT_CODE     AS HANDLER_CODE
    , NULL              AS TARGET_WORKSHEET
    , NULL              AS MENU_ORDER
    , NULL              AS EDIT_PARAMETERS
FROM
    ParameterValues s
    LEFT OUTER JOIN handlers t ON t.TABLE_NAME = s.TABLE_NAME
        AND COALESCE(s.PARAMETER_NAME, '')  = COALESCE(t.COLUMN_NAME, '')
        AND COALESCE(s.SELECT_SCHEMA, '')   = COALESCE(t.HANDLER_SCHEMA, '')
        AND COALESCE(s.SELECT_NAME, '')     = COALESCE(t.HANDLER_NAME, '')
        AND COALESCE(s.SELECT_TYPE, '')     = COALESCE(t.HANDLER_TYPE, '')
WHERE
    NOT s.TABLE_NAME IN ('ColumnTranslation', 'ObjectTranslation', 'TableFormats', 'EventHandlers', 'ParameterValues', 'QueryList')
    AND t.ID IS NULL;

-- print ColumnTranslation -> translations

INSERT INTO translations
    ( TABLE_SCHEMA
    , TABLE_NAME
    , COLUMN_NAME
    , LANGUAGE_NAME
    , TRANSLATED_NAME
    , TRANSLATED_DESC
    , TRANSLATED_COMMENT
    )
SELECT
    s.TABLE_SCHEMA
    , s.TABLE_NAME
    , NULL AS COLUMN_NAME
    , s.LANGUAGE_NAME
    , s.TRANSLATED_NAME
    , s.TRANSLATED_DESC
    , s.TRANSLATED_COMMENT
FROM
    ObjectTranslation s
    LEFT OUTER JOIN translations t ON t.TABLE_NAME = s.TABLE_NAME
        AND t.LANGUAGE_NAME = s.LANGUAGE_NAME AND t.COLUMN_NAME IS NULL
WHERE
    NOT s.TABLE_NAME IN ('ColumnTranslation', 'ObjectTranslation', 'TableFormats', 'EventHandlers', 'ParameterValues', 'QueryList')
    AND t.ID IS NULL;

-- print ObjectTranslation -> translations

INSERT INTO translations
    ( TABLE_SCHEMA
    , TABLE_NAME
    , COLUMN_NAME
    , LANGUAGE_NAME
    , TRANSLATED_NAME
    , TRANSLATED_DESC
    , TRANSLATED_COMMENT
    )
SELECT
    s.TABLE_SCHEMA
    , s.TABLE_NAME
    , s.COLUMN_NAME
    , s.LANGUAGE_NAME
    , s.TRANSLATED_NAME
    , s.TRANSLATED_DESC
    , NULL AS TRANSLATED_COMMENT
FROM
    ColumnTranslation s
    LEFT OUTER JOIN translations t ON t.TABLE_NAME = s.TABLE_NAME
        AND t.LANGUAGE_NAME = s.LANGUAGE_NAME AND t.COLUMN_NAME = s.COLUMN_NAME
WHERE
    NOT s.TABLE_NAME IN ('ColumnTranslation', 'ObjectTranslation', 'TableFormats', 'EventHandlers', 'ParameterValues', 'QueryList')
    AND t.ID IS NULL;

-- print TableFormats      -> formats

INSERT INTO formats
    ( TABLE_SCHEMA
    , TABLE_NAME
    , TABLE_EXCEL_FORMAT_XML
    )
SELECT
    s.TABLE_SCHEMA
    , s.TABLE_NAME
    , s.TABLE_EXCEL_FORMAT_XML
FROM
    TableFormats s
    LEFT OUTER JOIN formats t ON t.TABLE_NAME = s.TABLE_NAME
WHERE
    NOT s.TABLE_NAME IN ('ColumnTranslation', 'ObjectTranslation', 'TableFormats', 'EventHandlers', 'ParameterValues', 'QueryList')
    AND t.ID IS NULL;
